package kojaview.parts;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.annotation.PostConstruct;
import javax.inject.Inject;
import javax.inject.Named;

import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.Focus;
import org.eclipse.e4.ui.services.IServiceConstants;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.dom.AST;
import org.eclipse.jdt.core.dom.ASTParser;
import org.eclipse.jdt.core.dom.ASTRequestor;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.ISelectionProvider;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IPartListener2;
import org.eclipse.ui.IPartService;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchPartReference;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.texteditor.ITextEditor;

public class KojaView extends ViewPart implements IPartListener2, ISelectionChangedListener {
	private Label myLabelInView;
	private TreeViewer treev = null;
	private List<MethodDeclaration> methods = null;
	private MyLabelProvider labelProvider = new MyLabelProvider();
	private StringBuffer buffer = new StringBuffer();
	private Set<IEditorPart> setEditors = new HashSet<IEditorPart>();
	public KojaView() {
		init();
	}
	
	private void init() {
	}

	@PostConstruct
	public void createPartControl(Composite parent) {
		System.out.println("Enter in SampleE4View postConstruct");

		myLabelInView = new Label(parent, SWT.BORDER|SWT.WRAP);
		myLabelInView.setText("This is a sample E4 view");

		treev = new TreeViewer(parent, SWT.WRAP|SWT.MULTI|SWT.V_SCROLL|SWT.BORDER);
		treev.setContentProvider(new MyContentProvider());
		treev.setLabelProvider(new MyLabelProvider());
		
		// part listener setting
		IWorkbench workbench = PlatformUI.getWorkbench();
		IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
		IWorkbenchPage page = window.getActivePage();
		IPartListener2 l = this;
		page.addPartListener(l);
		
	}
	
	private void createNewEditorPart(IEditorPart editorPart) {
		setEditors.add(editorPart);
		if(editorPart instanceof ITextEditor) {
			ITextEditor textEditor = (ITextEditor) editorPart;
			ISelectionProvider sp = textEditor.getSelectionProvider();
			ISelectionChangedListener scl = this;			
			sp.addSelectionChangedListener(scl);
		}
	}
	
	private void doSetContent(IWorkbenchPartReference partRef) {
		if (partRef==null) return;
		final IWorkbenchPart part = partRef.getPart(false);
		
		if (part instanceof IEditorPart) {
			MyVisitor visitor = new MyVisitor();
			
			final IEditorPart editorPart = (IEditorPart) part;
			if(!setEditors.contains(editorPart)) {
				createNewEditorPart(editorPart);
			}
			IEditorInput editorInput = editorPart.getEditorInput();
			if (editorInput==null) return; // null check
			
			Object obj = editorInput.getAdapter(IJavaElement.class);
			if(obj instanceof ICompilationUnit) {
				ASTParser parser = ASTParser.newParser(AST.JLS10);
				ICompilationUnit units[] = { (ICompilationUnit)obj };
				String[] keys = new String[] {};
				parser.createASTs(units, keys, new ASTRequestor() {
					@Override
			           public final void acceptAST(final ICompilationUnit unit, final CompilationUnit node) {
						node.accept(visitor);
						methods = visitor.getMethods();
						treev.setInput(methods);
						treev.expandAll();
						buffer = new StringBuffer();
						for(MethodDeclaration method: methods) {
							buffer.append(labelProvider.getText(method));
							buffer.append('\n');
						}
						myLabelInView.setText(buffer.toString());
					}
				}, new NullProgressMonitor());
			}
			
		}
		
	}


	@Focus
	public void setFocus() {
		myLabelInView.setFocus();

	}

	/**
	 * This method is kept for E3 compatibility. You can remove it if you do not
	 * mix E3 and E4 code. <br/>
	 * With E4 code you will set directly the selection in ESelectionService and
	 * you do not receive a ISelection
	 * 
	 * @param s
	 *            the selection received from JFace (E3 mode)
	 */
	@Inject
	@Optional
	public void setSelection(@Named(IServiceConstants.ACTIVE_SELECTION) ISelection s) {
		if (s==null || s.isEmpty())
			return;

		if (s instanceof IStructuredSelection) {
			IStructuredSelection iss = (IStructuredSelection) s;
			if (iss.size() == 1)
				setSelection(iss.getFirstElement());
			else
				setSelection(iss.toArray());
		}
	}

	/**
	 * This method manages the selection of your current object. In this example
	 * we listen to a single Object (even the ISelection already captured in E3
	 * mode). <br/>
	 * You should change the parameter type of your received Object to manage
	 * your specific selection
	 * 
	 * @param o
	 *            : the current object received
	 */
	@Inject
	@Optional
	public void setSelection(@Named(IServiceConstants.ACTIVE_SELECTION) Object o) {

		// Remove the 2 following lines in pure E4 mode, keep them in mixed mode
		if (o instanceof ISelection) // Already captured
			return;

		// Test if label exists (inject methods are called before PostConstruct)
		if (myLabelInView != null)
			myLabelInView.setText("Current single selection class is : " + o.getClass());
	}

	/**
	 * This method manages the multiple selection of your current objects. <br/>
	 * You should change the parameter type of your array of Objects to manage
	 * your specific selection
	 * 
	 * @param o
	 *            : the current array of objects received in case of multiple selection
	 */
	@Inject
	@Optional
	public void setSelection(@Named(IServiceConstants.ACTIVE_SELECTION) Object[] selectedObjects) {

		// Test if label exists (inject methods are called before PostConstruct)
		if (myLabelInView != null)
			myLabelInView.setText("This is a multiple selection of " + selectedObjects.length + " objects");
	}
	
	// IPartListener2 overrides
	/**
	 * @override
	 * @param partRef
	 */
	public void partActivated(IWorkbenchPartReference partRef) {
		System.out.println("partActivated " + partRef.getPartName());
		doSetContent(partRef);
	}
	
	public void partBroughtToTop(IWorkbenchPartReference partRef) {
		System.out.println("partBroughtToTop " + partRef.getPartName());
		doSetContent(partRef);		
	}
	
    /**
     * Notifies this listener that the given part has been closed.
     * <p>
     * Note that if other perspectives in the same page share the view,
     * this notification is not sent.  It is only sent when the view
     * is being removed from the page entirely (it is being disposed).
     * </p>
     *
	 * @override
     * @param partRef the part that was closed
     * @see IWorkbenchPage#hideView
     */
	public void partClosed(IWorkbenchPartReference partRef) {
		System.out.println("partClosed " + partRef.getPartName());
//		doSetContent(partRef);		
	}

    /**
     * Notifies this listener that the given part has been deactivated.
     *
	 * @override
     * @param partRef the part that was deactivated
     * @see IWorkbenchPage#activate
     */
	public void partDeactivated(IWorkbenchPartReference partRef) {
		System.out.println("partDeactivated " + partRef.getPartName());
		if(partRef instanceof IEditorReference) {
			IEditorReference editorRef = (IEditorReference) partRef;
			setEditors.remove(editorRef.getEditor(true));
		}
//		doSetContent(partRef);		
	}

    /**
     * Notifies this listener that the given part has been opened.
     * <p>
     * Note that if other perspectives in the same page share the view,
     * this notification is not sent.  It is only sent when the view
     * is being newly opened in the page (it is being created).
     * </p>
     *
	 * @override
     * @param partRef the part that was opened
     * @see IWorkbenchPage#showView
     */
	public void partOpened(IWorkbenchPartReference partRef) {
		System.out.println("partOpened " + partRef.getPartName());
		doSetContent(partRef);		
	}

    /**
     * Notifies this listener that the given part is hidden or obscured by another part.
     *
	 * @override
     * @param partRef the part that is hidden or obscured by another part
     */
	public void partHidden(IWorkbenchPartReference partRef) {
		System.out.println("partHidden " + partRef.getPartName());
//		doSetContent(partRef);		
	}

    /**
     * Notifies this listener that the given part is visible.
     *
	 * @override
     * @param partRef the part that is visible
     */
	public void partVisible(IWorkbenchPartReference partRef) {
		System.out.println("partVisible " + partRef.getPartName());
		doSetContent(partRef);		
	}

    /**
     * Notifies this listener that the given part's input was changed.
     *
	 * @override
     * @param partRef the part whose input was changed
     */
	public void partInputChanged(IWorkbenchPartReference partRef) {
		System.out.println("partInputChanged " + partRef.getPartName());
		doSetContent(partRef);		
	}

	@Override
	public void selectionChanged(SelectionChangedEvent event) {
		// TODO Auto-generated method stub
		System.out.println("selectionChangedListener" + event.toString());
		IWorkbench workbench = PlatformUI.getWorkbench();
		IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
		IPartService service = window.getPartService();
		IWorkbenchPartReference partRef = service.getActivePartReference();
		doSetContent(partRef);
	}

}
