package kojaview.parts;

import java.util.Arrays;
import java.util.List;

import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.ISourceReference;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.Comment;
import org.eclipse.jdt.core.dom.CompilationUnit;
import org.eclipse.jdt.core.dom.IDocElement;
import org.eclipse.jdt.core.dom.Javadoc;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.Modifier;
import org.eclipse.jdt.core.dom.Name;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;
import org.eclipse.jdt.core.dom.TagElement;
import org.eclipse.jdt.core.dom.TextElement;
import org.eclipse.jdt.core.dom.Type;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.graphics.Image;

public class MyLabelProvider extends LabelProvider {
	StringBuffer buffer = null;
	/**
	 * @override
	 */
	public String getText(Object element) {
		if (element instanceof String) {
			return (String)element;
		} else if (element instanceof MethodDeclaration) {
			return ((MethodDeclaration)element).getName().toString();
		} else if (element instanceof Javadoc) {
			Javadoc doc = (Javadoc) element;
			List<TagElement> tagElements = doc.tags();
			buffer = new StringBuffer();
			for(TagElement tagElement: tagElements) {
				buffer.append(tagElement.getTagName() + " ");
				List<IDocElement> iDocElements = tagElement.fragments();
				for(IDocElement idocelement: iDocElements) {
					if (idocelement instanceof Name) {
						buffer.append(((Name) idocelement).getFullyQualifiedName());
					} else if(idocelement instanceof SimpleName) {
						buffer.append(((SimpleName) idocelement).getIdentifier());
					} else if(idocelement instanceof TextElement) {
						buffer.append(((TextElement) idocelement).getText());
					}
					buffer.append(' ');
				}
				buffer.append(',');
			}
			return buffer.toString();
		} else if (element instanceof Integer) {
			int flag = (Integer) element;
			switch(flag) {
			case Modifier.PRIVATE:
				return "□";
			case Modifier.STATIC:
				return "◯";
			case Modifier.PUBLIC:
				return "△";
			case Modifier.PROTECTED:
				return "◇";
			case Modifier.ABSTRACT:
				return "◎";
			default:
				return "none";
			}
		} else if (element instanceof Type) {
			return ((Type)element).toString();
		} else if (element instanceof List<?>) {
			List<Object> list = (List<Object>) element;
			if(list.get(0) instanceof SingleVariableDeclaration) {
				return "Return Types";
			} else if (list.get(0) instanceof Type) {
				return "Exceptions";
			}
		} else if (element instanceof SingleVariableDeclaration) {
			SingleVariableDeclaration var = (SingleVariableDeclaration) element;
			buffer = new StringBuffer();
			buffer.append(var.getType());
			buffer.append(' ');
			buffer.append(var.getName());
			return buffer.toString();
		}
		return "unknown";
	}
}
